// KreateCD - CD recording software for the K desktop environment
//
// 2000 by Alexander Feigl <Alexander.Feigl@gmx.de>
//
// This file is subject to the terms and conditions of the GNU General
// Public License.  See the file COPYING in the main directory of the
// KreateCD distribution for more details.

#include "TopWidget.h"

#include <kapp.h>
#include <klocale.h>
#include <kmenubar.h>
#include <kconfig.h>

#include <qkeycode.h>
#include <qrect.h>

#include "MainWindow.h"

TopWidget::TopWidget(const char *name):KTMainWindow(name) {
  QRect DefaultRect(100,20,500,430);
  KConfig *config;
  MainWindow *mainw;


#if QT_VERSION >= 200
  config=kapp->config();
#else
  config=kapp->getConfig();
#endif



  mainw=new MainWindow(this);
  setView(mainw);

  MWMenu=new KMenuBar(this,"kreatecd menu");
  QPopupMenu *filemenu=new QPopupMenu;
  filemenu->insertItem(i18n("New project..."),mainw,SLOT(newProject()));
  filemenu->insertItem(i18n("Open project..."),mainw,SLOT(openProject()));
  filemenu->insertSeparator();
  filemenu->insertItem(i18n("Save project"),mainw,SLOT(saveProject()));
  filemenu->insertItem(i18n("Save project as..."),mainw,SLOT(saveProjectAs()));
  filemenu->insertSeparator();
  filemenu->insertItem(i18n("Quit"),this,SLOT(leaveMe()));
  MWMenu->insertItem(i18n("File"),filemenu);
  QPopupMenu *toolsmenu=new QPopupMenu;
  toolsmenu->insertItem(i18n("&Copy CDROM..."),mainw,SLOT(setupCopy()));
  QPopupMenu *audiomenu=new QPopupMenu;
  audiomenu->insertItem(i18n("Normalize to maximum"),mainw,SLOT(normalizeMax()));
  audiomenu->insertItem(i18n("Normalize all to same"),mainw,SLOT(normalizeAllSame()));
  toolsmenu->insertItem(i18n("Audio"),audiomenu);
  QPopupMenu *blankmenu=new QPopupMenu;
  blankmenu->insertItem(i18n("blank entire disk"),mainw,SLOT(blankAll()));
  blankmenu->insertItem(i18n("fast blanking"),mainw,SLOT(blankFast()));
  blankmenu->insertItem(i18n("blank track"),mainw,SLOT(blankTrack()));
  blankmenu->insertItem(i18n("unreserve track"),mainw,SLOT(blankUnreserve()));
  blankmenu->insertItem(i18n("blank track tail"),mainw,SLOT(blankTail()));
  blankmenu->insertItem(i18n("unclose last session"),mainw,SLOT(blankUnclose()));
  blankmenu->insertItem(i18n("blank last session"),mainw,SLOT(blankSession()));
  toolsmenu->insertItem(i18n("CD-RW blanking"),blankmenu);

  MWMenu->insertItem(i18n("&Tools"),toolsmenu);
  QPopupMenu *configmenu=new QPopupMenu;
  configmenu->insertItem(i18n("SCSI..."),mainw,SLOT(configSCSI()));
  configmenu->insertItem(i18n("Paths..."),mainw,SLOT(configPath()));
  configmenu->insertItem(i18n("ISO 9660..."),mainw,SLOT(configISO()));
  MWMenu->insertItem(i18n("Configure"),configmenu);
  MWMenu->insertSeparator();
  QPopupMenu *helpmenu=new QPopupMenu;
  helpmenu->insertItem(i18n("Contents"),mainw,SLOT(helpContents()),Key_F1,-1);
  helpmenu->insertSeparator();
  helpmenu->insertItem(i18n("About KreateCD..."),mainw,SLOT(helpAboutMe()));
  MWMenu->insertItem(i18n("Help"),helpmenu);

  setMenu(MWMenu);

  config->setGroup("Geometry");
  this->setGeometry(config->readRectEntry("MainWindowGeo",&DefaultRect));
}


TopWidget::~TopWidget()
 {
  KConfig *config;
  QRect fgeo;

  fgeo=this->geometry();
  QRect SaveRect(fgeo.topLeft(),fgeo.size());

#if QT_VERSION >= 200
  config=kapp->config();
#else
  config=kapp->getConfig();
#endif
  config->setGroup("Geometry");
  config->writeEntry("MainWindowGeo",SaveRect);
 }

void TopWidget::leaveMe(void)
 {
   this->close();
 }
